package electric_vehicles;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import electric_vehicles.Agents.AgentLease;
import electric_vehicles.Agents.AgentNew;
import electric_vehicles.Agents.AgentOccasion;
import electric_vehicles.Enums.ChargeLocation;
import electric_vehicles.Enums.FuelType;
import electric_vehicles.Enums.NeedWeight;
import electric_vehicles.Enums.SocialWeight;
import electric_vehicles.Media.Media;
import electric_vehicles.Media.RandomWriter;
import electric_vehicles.Media.TruthfulWriter;
import electric_vehicles.Scenarios.Scenarios;
import electric_vehicles.VehicleInfrastructure.Mechanic;
import electric_vehicles.VehicleInfrastructure.Tax;
import electric_vehicles.VehicleInfrastructure.Marketplace;
import electric_vehicles.VehicleInfrastructure.RechargeStation;
import electric_vehicles.Vehicles.Battery;
import electric_vehicles.Vehicles.ElectricVehicle;
import electric_vehicles.Vehicles.GasolineVehicle;
import electric_vehicles.Vehicles.HybridVehicle;
import repast.simphony.context.Context;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.essentials.RepastEssentials;
import repast.simphony.random.RandomHelper;


/**
 * @author Ayla
 *
 */

public class ConsumatBuilder implements ContextBuilder<Object> 
{
	private int total_agents;
	private Map<Integer, Integer> priceRangeOcc;
	
	public Context<Object> build(Context<Object> context) 
	{
		// Load scenario
		total_agents = 0;
		priceRangeOcc = new HashMap<Integer, Integer>();
		context.setId("electric_vehicles");
		
		// Initialize static variables
		context.add(new Tax());
		context.add(new Battery());
		context.add(new RechargeStation());
		context.add(new Mechanic());
		context.add(new Marketplace());
		
		// Create the common marketplace where cars are bought and sold
		initializeMarketplace();		
		
		// Create the agents
		initializeAgents(context); 
				
		Marketplace.initialiseOccasionMarket(total_agents);
		RechargeStation.setHomeChargeProb(total_agents);

		
		// Add the media and load the scenario to the context
		context.add(new Media());
		context.add(new Scenarios());
//		context.add(new Statistics()); // writes various output metrics to files for further analysis
		initializeMediaWriter(context);
				
		return context;
	}
	
	public void initializeMarketplace()
	{
		// Open and read data file
		File data_file = new File("data/vehicles.csv");
		BufferedReader reader = null;	
		try
		{
			reader = new BufferedReader(new FileReader(data_file));
			String row = null;
			
			// Parse data labels
			List<String> labels = Arrays.asList(reader.readLine().split(";"));
			
			// Parse vehicles
			while ((row = reader.readLine()) != null)
			{
				// Split data from this vehicle
				List<String> data_elements = Arrays.asList(row.split(";"));
						
				// Obtain general variable values
				FuelType fuelType = FuelType.fromString(data_elements.get(labels.indexOf("Type")));
				String model = data_elements.get(labels.indexOf("Model"));
				float price = Float.parseFloat(data_elements.get(labels.indexOf("Price")));
				int emissions = Integer.parseInt(data_elements.get(labels.indexOf("Emissions")));

				
				// Create EV
				if (fuelType == FuelType.ELECTRIC)
				{
					int battery_capacity = Integer.parseInt(data_elements.get(labels.indexOf("BatteryCapacity")));
					Marketplace.introduceCar(new ElectricVehicle(fuelType, model, (int)RepastEssentials.GetTickCount() + RandomHelper.nextIntFromTo(-52, 0), price, battery_capacity * Battery.getKWhCosts(), emissions, battery_capacity));
				}
				// Create hybrid
				else if (fuelType == FuelType.HYBRID)
				{
					int range = Integer.parseInt(data_elements.get(labels.indexOf("Range")));
					int battery_capacity = Integer.parseInt(data_elements.get(labels.indexOf("BatteryCapacity")));
					Marketplace.introduceCar(new HybridVehicle(fuelType, model, (int)RepastEssentials.GetTickCount() + RandomHelper.nextIntFromTo(-52, 0), price,  battery_capacity * Battery.getKWhCosts(), range, emissions, battery_capacity));
				}
				// Create gasoline car
				else if(fuelType == FuelType.GASOLINE)
				{
					int range = Integer.parseInt(data_elements.get(labels.indexOf("Range")));
					Marketplace.introduceCar(new GasolineVehicle(fuelType, model, (int)RepastEssentials.GetTickCount() + RandomHelper.nextIntFromTo(-52, 0), price, range, emissions));
				}
			}
		} 
		catch (FileNotFoundException e) 
		{
		    e.printStackTrace();
		}
		catch (IOException e) 
		{
		    e.printStackTrace();
		} 
		finally 
		{
		    try 
		    {
		        if (reader != null) 
		            reader.close();
		    } 
		    catch (IOException e) 
		    {}
		}
	}

	public void initializeAgents(Context<Object> context)	
	{		
		// Open and read data file
		File data_file = new File("data/agents.csv");
		BufferedReader reader = null;	
		try
		{
			reader = new BufferedReader(new FileReader(data_file));
			String row = null;
			
			// Parse data labels
			List<String> labels = Arrays.asList(reader.readLine().split(";"));
			
			// Parse participants
			while ((row = reader.readLine()) != null)
			{
				
				// Split data from this participant
				List<String> data_elements = Arrays.asList(row.split(";"));
				// Skip participants with incomplete data
				if (data_elements.get(labels.indexOf("WHENbought")).isEmpty() || Integer.parseInt(data_elements.get(labels.indexOf("INCOMEhsh"))) == 11)
					continue;
					
				// ------------- Obtain variable values ------------- 
				int id = Integer.parseInt(data_elements.get(labels.indexOf("Number")));
				
				// Personality variable values
				Map<NeedWeight, Float> need_weights = converseNeedWeights(Integer.parseInt(data_elements.get(labels.indexOf("W_7"))), Integer.parseInt(data_elements.get(labels.indexOf("W_4"))),Integer.parseInt(data_elements.get(labels.indexOf("W_10"))),
						Integer.parseInt(data_elements.get(labels.indexOf("W_1"))), Integer.parseInt(data_elements.get(labels.indexOf("W_9"))), 
						Integer.parseInt(data_elements.get(labels.indexOf("W_3"))), Integer.parseInt(data_elements.get(labels.indexOf("W_8"))),
						Integer.parseInt(data_elements.get(labels.indexOf("W_11"))), Integer.parseInt(data_elements.get(labels.indexOf("W_14"))));
				float ambition = converseAmbition(Integer.parseInt(data_elements.get(labels.indexOf("W_16"))));
				float uncertainty_tolerance = converseUncertaintyTolerance(Integer.parseInt(data_elements.get(labels.indexOf("C_SEG"))));
				float expertise = converseExpertise(Integer.parseInt(data_elements.get(labels.indexOf("KNOW_1"))), Integer.parseInt(data_elements.get(labels.indexOf("KNOW_2"))),
						Integer.parseInt(data_elements.get(labels.indexOf("KNOW_3"))));
				
				// Needs variable values
				// Existence need
				Map<Integer,List<Integer>> distance_freq = constructKMTable(new LinkedList<Integer>(Arrays.asList(Integer.parseInt(data_elements.get(labels.indexOf("KMtable_8"))), 
						Integer.parseInt(data_elements.get(labels.indexOf("KMtable_9"))), Integer.parseInt(data_elements.get(labels.indexOf("KMtable_10"))),
						Integer.parseInt(data_elements.get(labels.indexOf("KMtable_11"))), Integer.parseInt(data_elements.get(labels.indexOf("KMtable_12"))),
						Integer.parseInt(data_elements.get(labels.indexOf("KMtable_13"))), Integer.parseInt(data_elements.get(labels.indexOf("KMtable_14"))))));
				if (distance_freq.get(1).isEmpty()) // Inconsistent drive table
					continue;
				int min_range = Integer.parseInt(data_elements.get(labels.indexOf("ACCminR")));
				Map<ChargeLocation, Integer> max_charge_times = converseChargeTimes(Integer.parseInt(data_elements.get(labels.indexOf("tableCHt_1_1"))),
						Integer.parseInt(data_elements.get(labels.indexOf("tableCHt_1_2"))), Integer.parseInt(data_elements.get(labels.indexOf("tableCHt_1_3"))),
						Integer.parseInt(data_elements.get(labels.indexOf("tableCHt_1_4"))), Integer.parseInt(data_elements.get(labels.indexOf("tableCHt_1_5"))));
				
				// Social need
				Map<SocialWeight, Float> social_weights = converseSocialWeights(Integer.parseInt(data_elements.get(labels.indexOf("INNOV_1"))), 
						Integer.parseInt(data_elements.get(labels.indexOf("INNOV_3"))), Integer.parseInt(data_elements.get(labels.indexOf("INNOV_7"))));
				
				// Preference need
				float max_emissions = converseMaxEmissions(Integer.parseInt(data_elements.get(labels.indexOf("SEV_2"))), Integer.parseInt(data_elements.get(labels.indexOf("VLN_2"))),
						Integer.parseInt(data_elements.get(labels.indexOf("SEV_3"))), Integer.parseInt(data_elements.get(labels.indexOf("VLN_3"))), 
						Integer.parseInt(data_elements.get(labels.indexOf("SEV_4"))), Integer.parseInt(data_elements.get(labels.indexOf("VLN_4"))));
				float enjoy_innovation = converseEnjoymentInnovation(Integer.parseInt(data_elements.get(labels.indexOf("INNOV_2"))), Integer.parseInt(data_elements.get(labels.indexOf("INNOV_5"))),
						Integer.parseInt(data_elements.get(labels.indexOf("INNOV_6"))));
				
				// Abilities variable values
				int age = 2012 - Integer.parseInt("19" + data_elements.get(labels.indexOf("YofBIRTH")));
				int location = conversePostcode(Integer.parseInt(data_elements.get(labels.indexOf("postcode")))); 
				int income_category = Integer.parseInt(data_elements.get(labels.indexOf("INCOMEhsh")));
				int income = converseIncome(income_category);
				boolean homecharge = data_elements.get(labels.indexOf("ownPARK")).equals("1") ? true : false;
				float recharge_moment = converseRechargeMoment(Integer.parseInt(data_elements.get(labels.indexOf("TANKgedrag")))); 
				boolean buys_new = data_elements.get(labels.indexOf("EXnew_used")).equals("1") ? true : false;
				if (Integer.parseInt(data_elements.get(labels.indexOf("currFUEL"))) == 3 || Integer.parseInt(data_elements.get(labels.indexOf("currFUEL"))) == 5 || Integer.parseInt(data_elements.get(labels.indexOf("currFUEL"))) == 6)
					continue;
				FuelType current_fuel_type = converseFuelType(Integer.parseInt(data_elements.get(labels.indexOf("currFUEL"))));
				if (Integer.parseInt(data_elements.get(labels.indexOf("currentKM"))) == 11 && data_elements.get(labels.indexOf("currentKM_TEXT")).matches(" "))
					continue;
				// If respondent owns more than 1 car, get info from most used car.
				int n_cars = Integer.parseInt(data_elements.get(labels.indexOf("Nautos"))); 
				boolean lease_car = false;
				int current_vehicle_age = 0;
				switch (n_cars)	// Does agent drive a lease car and what is build year of car?
				{
					case 1: System.out.println("WARNING: RESPONDENT HAS ZERO CARS BUT IS STILL INITIALISED");
							break;
					case 2: lease_car = data_elements.get(labels.indexOf("car_spec1_1_1")).equals("1") ? true : false;
							current_vehicle_age = 2012 - Integer.parseInt(data_elements.get(labels.indexOf("car_spec1_4_1")));
							break;
					case 3: String label = data_elements.get(labels.indexOf("Udrive123"));
							lease_car = data_elements.get(labels.indexOf("car_spec23_1_" + label)).equals("1") ? true : false;
							current_vehicle_age = 2012 - Integer.parseInt(data_elements.get(labels.indexOf("car_spec23_4_" + label)));
				}
				int vehicle_in_possesion_age = 2012 - Integer.parseInt(data_elements.get(labels.indexOf("WHENbought")));
				int yearly_km_category = Integer.parseInt(data_elements.get(labels.indexOf("currentKM"))) <= 10 ? Integer.parseInt(data_elements.get(labels.indexOf("currentKM"))) : 10;
				int yearly_km_text = Integer.parseInt(data_elements.get(labels.indexOf("currentKM"))) <= 10 ? 0 : Integer.parseInt(data_elements.get(labels.indexOf("currentKM_TEXT")));
				int yearly_km = converseKmCount(2012, Integer.parseInt(data_elements.get(labels.indexOf("currentKM"))), yearly_km_text);			
				List<Float> car_costs_range = buys_new ? converseMaxPriceNewVehicle(Integer.parseInt(data_elements.get(labels.indexOf("PRICEnew")))) : converseMaxPriceOccasionVehicle(Integer.parseInt(data_elements.get(labels.indexOf("PRICEused"))));							
				
				++total_agents;
				
				// Create agent
				if (lease_car) 
					context.add(new AgentLease(id, need_weights, ambition, uncertainty_tolerance, expertise, distance_freq, min_range, max_charge_times, 
							social_weights, max_emissions, enjoy_innovation, income_category, income, age, location, homecharge, recharge_moment,
							current_fuel_type, current_vehicle_age, vehicle_in_possesion_age, yearly_km_category, yearly_km, car_costs_range, buys_new));
				else if (buys_new) 
					context.add(new AgentNew(id, need_weights, ambition, uncertainty_tolerance, expertise, distance_freq, min_range, max_charge_times, 
							social_weights, max_emissions, enjoy_innovation, income_category, income, age, location, homecharge, recharge_moment,
							current_fuel_type, current_vehicle_age, vehicle_in_possesion_age, yearly_km_category, yearly_km, car_costs_range, buys_new));
				else
					context.add(new AgentOccasion(id, need_weights, ambition, uncertainty_tolerance, expertise, distance_freq, min_range, max_charge_times, 
							social_weights, max_emissions, enjoy_innovation, income_category, income, age, location, homecharge, recharge_moment, 
							current_fuel_type, current_vehicle_age, vehicle_in_possesion_age, yearly_km_category, yearly_km, car_costs_range, buys_new)); 
			}
		}
		catch (FileNotFoundException e) 
		{
		    e.printStackTrace();
		}
		catch (IOException e) 
		{
		    e.printStackTrace();
		} 
		finally 
		{
		    try 
		    {
		        if (reader != null) 
		            reader.close();
		    } 
		    catch (IOException e) 
		    {}
		}
	}
	
	/* Creates a weight set of the importance of the four different Consumat needs. 
	 */	
	public Map<NeedWeight, Float> converseNeedWeights(int costs, int func1, int func2, int soc1, int soc2, int soc3, int soc4, int pref1, int pref2)
	{
		Map<NeedWeight, Float> need_weights = new HashMap<NeedWeight,Float>();
		
		// Combine four measures of preference weight from the survey into one measure
		float cost = (float)costs;
		float functionality = (float)(func1 + func2)/2;
		float social = (float)(soc1 + soc2)/2;
		float antisocial = (float)(soc3 + soc4)/2;
		float preferences = (float)(pref1 + pref2)/2;
		
		// Add weights as a proportion of each other to the list
		float total_weight = social >= antisocial ? costs + functionality + social + preferences : costs + functionality + antisocial + preferences;
				
		need_weights.put(NeedWeight.COSTS, (float)cost/total_weight);
		need_weights.put(NeedWeight.FUNCTIONALITY, (float)functionality/total_weight);
		need_weights.put(NeedWeight.SOCIAL, social >= antisocial ? (float)social/total_weight : (float)antisocial/total_weight) ;		
		need_weights.put(NeedWeight.PREFERENCES, (float)preferences/total_weight);	
		
		return need_weights;
	}
	
	/* Converses survey answer for ambition into a usable fraction.
	 * High scores on ambition result in a high ambition value.
	 */
	public float converseAmbition(int surveyValue)
	{
		switch(surveyValue)
		{
			case 1:		return 0f;
			case 2: 	return 0.2f;
			case 3: 	return 0.3f;
			case 4: 	return 0.4f;
			case 5: 	return 0.5f;
			case 6: 	return 0.6f;
			case 7: 	return 0.7f;
			case 8: 	return 0.8f;
			case 9: 	return 0.9f;			
			default: 	System.out.println("WARNING: INVALID AMBITION SURVEY VALUE");
						return 0f;
		}
	}

	/* Converses survey answer for Roger's consumer segment into a measure
	 * of uncertainty tolerance. The more risk-taking, the higher the uncertainty
	 * tolerance.
	 */
	public float converseUncertaintyTolerance(int surveyValue)
	{
		switch(surveyValue)
		{
			case 1:		return 1f;
			case 2: 	return 0.8f;
			case 3: 	return 0.6f;
			case 4: 	return 0.4f;
			case 5: 	return 0.2f;			
			default: 	System.out.println("WARNING: INVALID UNCERTAINTY TOLERANCE SURVEY VALUE");
						return 0f;
		}		
	}
	
	/* Converses survey answer for knowledge of alternative fuels into a usable fraction.
	 * High scores on knowledge of alternative fuels result in a high expertise value.
	 */
	public float converseExpertise(int exp1, int exp2, int exp3)
	{
		return (float)(exp1 + exp2 + exp3)/(3*6);
	}

	/* Converses survey answers for the frequency that different km ranges are driven to 
	 * a map. (Answers are 0: never. 1: once a week. 2: once a month, 3: once a year).
	 * Map uses the answers as keys, and all the distances that adhere to these answers as
	 * a list of values.
	 * Survey answers actually range between 1 and 4 but the above values are more intuitive.
	 */
	public Map<Integer,List<Integer>> constructKMTable(List<Integer> frequencies)
	{
		if (frequencies.size() != 7)
		{
			System.out.println("WARNING: INSUFFICIENT KM TABLE DATA");
			return null;
		}
		
		// Create map and possible keys
		Map<Integer,List<Integer>> km_table = new HashMap<Integer, List<Integer>>(); 
		for (int i = 0; i < 4; i++)
			km_table.put(i, new LinkedList<Integer>());
		
		// Add each distance range to corresponding keys
		km_table.get(frequencies.get(0)-1).add(0);
		km_table.get(frequencies.get(1)-1).add(10);
		km_table.get(frequencies.get(2)-1).add(50);
		km_table.get(frequencies.get(3)-1).add(100);
		km_table.get(frequencies.get(4)-1).add(200);
		km_table.get(frequencies.get(5)-1).add(500);
		km_table.get(frequencies.get(6)-1).add(1000);
		
		return km_table;
	}
	
	/* Stores the survey answers for the maximum amount of time that the car should be allowed
	 * to charge at different locations in a map. Before storage, the values are converted to
	 * their corresponding minutes (i.e. a value of '1' is conversed to '15' (minutes).
	 */
	public Map<ChargeLocation,Integer> converseChargeTimes(int home, int work, int garage, int supermarket, int highway)
	{
		Map<ChargeLocation, Integer> charge_times = new HashMap<ChargeLocation, Integer>();
		
		charge_times.put(ChargeLocation.HOME, coverseChargeValueToMinutes(home));
		charge_times.put(ChargeLocation.WORK, coverseChargeValueToMinutes(work));
		charge_times.put(ChargeLocation.HIGHWAY, coverseChargeValueToMinutes(highway));
				
		return charge_times;
	}
	
	/* Converts the values of the maximum charge times to their actual duration in minutes
	 */
	private int coverseChargeValueToMinutes(int chargeValue)
	{
		switch (chargeValue)
		{
			case 1:		return 15;
			case 2: 	return 30;
			case 3: 	return 60;
			case 4: 	return 240;
			case 5: 	return 480;
			case 6: 	return 720;
			case 7:		return 0;
			default:	System.out.println("WARNING: INVALID MAX CHARGE TIME SURVEY VALUE");
						return 0;
		}
	}
	
	/* Converses survey answers for anti-conformity and superiority desires to 
	 * a weight set for the social need. Also adds a random value for the 
	 * conformity desire.
	 */	
	public Map<SocialWeight, Float> converseSocialWeights(int anticonformity, int superiority1, int superiority2)
	{
		Map<SocialWeight, Float> social_weights = new HashMap<SocialWeight,Float>();
		
		// Conformity measure is not in the survey. Use gaussian distribution
		RandomHelper.createNormal(3.5, 0.85);
		float conformity = (float)Math.max(1, Math.min(6, RandomHelper.getNormal().nextDouble()));
		
		// Combine two measures of superiority from the survey into one measure
		float superiority = (superiority1 + superiority2)/2;
				
		// Add weights as a proportion of each other to the list
		float total_weight = conformity + anticonformity + superiority;
		social_weights.put(SocialWeight.CONFORMITY, (float)conformity / total_weight );
		social_weights.put(SocialWeight.ANTI_CONFORMITY, (float)anticonformity / total_weight );
		social_weights.put(SocialWeight.SUPERIORITY, (float)superiority / total_weight );		
		
		return social_weights;
	}
	
	public float converseFromSixToOneScale(int envValue)
	{
		switch (envValue)
		{
			case 1:		return 0;
			case 2: 	return 0.2f;
			case 3: 	return 0.4f;
			case 4: 	return 0.6f;
			case 5: 	return 0.8f;
			case 6: 	return 1f;
			default:	System.out.println("WARNING: INVALID ENVIRONMENTAL CONSCIOUSNESS SURVEY VALUE");
						return 0f;
		}		
	}
	
	public float converseMaxEmissions(int sev2, int vln2, int sev3, int vln3, int sev4, int vln4)
	{
		return (1 - ((converseFromSixToOneScale(sev2) * converseFromSixToOneScale(vln2) + 
				converseFromSixToOneScale(sev3) * converseFromSixToOneScale(vln3) + 
				converseFromSixToOneScale(sev4) * converseFromSixToOneScale(vln4)) / 3 ) ) * 250;
	}
	
	public float converseEnjoymentInnovation(int enjoy1, int enjoy2, int enjoy3)
	{
		return (converseFromSixToOneScale(enjoy1) + converseFromSixToOneScale(enjoy2) + converseFromSixToOneScale(enjoy3)) / 3;
	}
	
	public int converseIncome(int income)
	{
		switch (income)
		{
			case 1:		return RandomHelper.nextIntFromTo(1000,15000);
			case 2:		return 17500;
			case 3:		return 22500;
			case 4:		return 27500;
			case 5:		return 35000;
			case 6:		return 45000;
			case 7:		return 55000;
			case 8:		return 65000;
			case 9:		return 82500;
			case 10:	return RandomHelper.nextIntFromTo(95000,400000);
			default:	return 0;
		}
	}
	
	public int conversePostcode(int postcode)
	{
		return (int)Math.floor(postcode/1000);
	}
	
	public float converseRechargeMoment(int moment)
	{
		switch (moment)
		{
			case 1: 	return 0.5f;
			case 2:		return 0.25f;
			case 3:		return 0.1f;
			default:	System.out.println("ERROR: Invalid recharge moment given.");
						return 1;
		}
	}
	
	public FuelType converseFuelType(int fuelType)
	{
		if (fuelType == 4)
			return FuelType.HYBRID;
		else if (fuelType == 6)
			return FuelType.ELECTRIC;

		return FuelType.GASOLINE;
	}
	
	public int converseKmCount(int buy_year, int km_per_year, int km_per_year_text)
	{
		int owned_years = 2013 - buy_year;
		switch (km_per_year)
		{
			case 1:		return 2500 * owned_years;
			case 2: 	return 7500 * owned_years;
			case 3: 	return 12500 * owned_years;
			case 4: 	return 17500 * owned_years;
			case 5: 	return 22500 * owned_years;
			case 6: 	return 27500 * owned_years;
			case 7: 	return 32500 * owned_years;
			case 8: 	return 37500 * owned_years;
			case 9: 	return 42500 * owned_years;
			case 10: 	return 47500 * owned_years;
			case 11: 	return km_per_year_text * owned_years;
			default:	System.out.println("WARNING: INVALID CURRENT KM COUNT VALUE");
						return 0;
		}	
	}
	
	public List<Float> converseMaxPriceNewVehicle(int price)
	{
		List<Float> car_costs_range = new LinkedList<Float>();
		
		switch (price)
		{
			case 1:		car_costs_range.add(0.0f);
						car_costs_range.add(10000.0f);
						break;
			case 2: 	car_costs_range.add(10000.0f);
						car_costs_range.add(15000.0f);
						break;
			case 3: 	car_costs_range.add(15000.0f);
						car_costs_range.add(20000.0f);
						break;
			case 4: 	car_costs_range.add(20000.0f);
						car_costs_range.add(25000.0f);
						break;
			case 5: 	car_costs_range.add(25000.0f);
						car_costs_range.add(30000.0f);
						break;
			case 6: 	car_costs_range.add(30000.0f);
						car_costs_range.add(35000.0f);
						break;
			case 7: 	car_costs_range.add(35000.0f);
						car_costs_range.add(40000.0f);
						break;
			case 8: 	car_costs_range.add(40000.0f);
						car_costs_range.add(45000.0f);
						break;
			case 9: 	car_costs_range.add(45000.0f);
						car_costs_range.add(50000.0f);
						break;
			case 10: 	car_costs_range.add(50000.0f);
						car_costs_range.add(150000.0f);
						break;
			default:	System.out.println("WARNING: INVALID MAX PURCHASE PRICE FOR NEW VEHICLES");
						break;
		}	
		
		return car_costs_range;
	}
	
	public List<Float> converseMaxPriceOccasionVehicle(int price)
	{
		List<Float> car_costs_range = new LinkedList<Float>();
		
		if (!priceRangeOcc.containsKey(price))
			priceRangeOcc.put(price, 0);
		priceRangeOcc.put(price, priceRangeOcc.get(price)+1);
		
		switch (price)
		{
			case 1:		car_costs_range.add(0.0f);
						car_costs_range.add(5000.0f);
						break;
			case 2: 	car_costs_range.add(5000.0f);
						car_costs_range.add(7500.0f);
						break;
			case 3: 	car_costs_range.add(7500.0f);
						car_costs_range.add(10000.0f);
						break;
			case 4: 	car_costs_range.add(10000.0f);
						car_costs_range.add(12500.0f);
						break;
			case 5: 	car_costs_range.add(12500.0f);
						car_costs_range.add(15000.0f);
						break;
			case 6: 	car_costs_range.add(15000.0f);
						car_costs_range.add(17500.0f);
						break;
			case 7: 	car_costs_range.add(17500.0f);
						car_costs_range.add(20000.0f);
						break;
			case 8: 	car_costs_range.add(20000.0f);
						car_costs_range.add(22500.0f);
						break;
			case 9: 	car_costs_range.add(22500.0f);
						car_costs_range.add(25000.0f);
						break;
			case 10: 	car_costs_range.add(25000.0f);
						car_costs_range.add(50000.0f);
						break;
			default:	System.out.println("WARNING: INVALID MAX PURCHASE PRICE FOR NEW VEHICLES");
						break;
		}	
		
		return car_costs_range;
	}
	
	public void initializeMediaWriter(Context<Object> context)
	{
		context.add(new RandomWriter());
	}
	
}
