/**
 * 
 */
package electric_vehicles.Memory;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import electric_vehicles.Enums.FuelType;
import electric_vehicles.Enums.SocialWeight;


/**
 * @author Ayla
 *
 */
public class MemoryOthers {

	// Memory structure of (agent id : memory element)
	private Map<Integer, MemoryElement> d_memory;
	private int d_informationCount;
	private int d_max_informationCount;
	private Map<FuelType, List<Integer>> d_fuelTypeMaxOwned;
	
	// Constructor
	public MemoryOthers()
	{
		d_memory = new HashMap<Integer, MemoryElement>();
		d_informationCount = 0;
		d_max_informationCount = 1;
		d_fuelTypeMaxOwned = new HashMap<FuelType, List<Integer>>();
		d_fuelTypeMaxOwned.put(FuelType.ELECTRIC, new LinkedList<Integer>());
		d_fuelTypeMaxOwned.put(FuelType.HYBRID, new LinkedList<Integer>());
		d_fuelTypeMaxOwned.put(FuelType.GASOLINE, new LinkedList<Integer>());
	}
	
	// Getters
	public int getInformationCount()				{ return d_informationCount; }
	public int getMaxInformationCount()				{ return d_max_informationCount; }
	public int getMemorySize()						{ return d_memory.size(); }
	
	/* After communicating with another agent, all relevant information is updated and stored
	 */
	public void addMemory(int agent_id, VehicleConcept vehicle, float satisfaction, int information_count, float normative_weight) 					
	{ 		
		// Did agent switch fuel type since last conversation?
		if (d_memory.containsKey(agent_id) && d_memory.get(agent_id).getOwnedVehicle().getFuelType() != vehicle.getFuelType())
		{
			FuelType new_ft = vehicle.getFuelType();
			if (!d_fuelTypeMaxOwned.get(new_ft).contains(agent_id))
				d_fuelTypeMaxOwned.get(new_ft).add(agent_id);
		}
		// First conversation with this agent? 
		if (!d_memory.containsKey(agent_id))
		{
			d_memory.put(agent_id, new MemoryElement(agent_id));
			d_fuelTypeMaxOwned.get(vehicle.getFuelType()).add(agent_id);
		}
			
		// Other agent biggest expert until now?
		if (information_count > d_max_informationCount || d_max_informationCount == 0)
			d_max_informationCount = information_count;
		
		// Store information
		d_memory.get(agent_id).updateInformation(vehicle, satisfaction, normative_weight);
		
	}
	
	/* After each discussed vehicle and each seen advertisement, agent increases its
	 * information count and becomes more 'expert' like.
	 */
	public void increaseInformationCount(int count)
	{
		d_informationCount += count;
		if (d_informationCount > d_max_informationCount)
			d_max_informationCount = d_informationCount;
	}
	
	/* Compute statistics from knowledge in memory. 
	 */
	public Map<SocialWeight, Float> getSocialSatisfactionFactors(VehicleConcept vehicle, float own_satisfaction)
	{
		Map<SocialWeight, Float> stats = new HashMap<SocialWeight, Float>();
		
		// Agent proportion with same fuel type (conformity)
		float count_same_fuel = 0;
		
		// Agent proportion with lower existence satisfaction (superiority)
		float count_exis_satisfaction = 0;
		
		float total_agents = 0;
				
		for(MemoryElement agent_memory : d_memory.values())
		{			
			// Same fuel types? ((anti-)conformity)
			if (agent_memory.getOwnedVehicle().getFuelType() == vehicle.getFuelType())
				count_same_fuel += agent_memory.getNormativeWeight();
				//++count_same_fuel;

			// Lower satisfaction? (superiority)
			if (agent_memory.getSatisfaction() < own_satisfaction)
				count_exis_satisfaction += agent_memory.getNormativeWeight();
				//++count_exis_satisfaction;
			
			total_agents += agent_memory.getNormativeWeight();
			//++total_agents;
		}
		
		// Proportion of agents with same fuel type
		stats.put(SocialWeight.CONFORMITY, Float.isNaN(count_same_fuel/total_agents) ? 1 : count_same_fuel/total_agents);
		// Proportion of agents with different vehicle model
		stats.put(SocialWeight.ANTI_CONFORMITY, Float.isNaN((total_agents - count_same_fuel)/total_agents) ? 1 : (total_agents - count_same_fuel)/total_agents);
		// Proportion of agents with lower satisfaction
		stats.put(SocialWeight.SUPERIORITY, Float.isNaN(count_exis_satisfaction/total_agents) ? 1 : count_exis_satisfaction/total_agents);
		
		return stats;
	}
	
	public float getSocialUncertainty(FuelType ft)
	{		
		if (d_fuelTypeMaxOwned.get(ft).size() == 0)
			return 1;
		
		float same_fueltype = 0;
		for(MemoryElement agent_memory : d_memory.values())
			if (agent_memory.getOwnedVehicle().getFuelType() == ft)
				++same_fueltype;
		
		return 1 - ( same_fueltype / d_fuelTypeMaxOwned.get(ft).size() );
	}
	
}
