/**
 * 
 */
package electric_vehicles.Memory;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import repast.simphony.random.RandomHelper;
import electric_vehicles.Enums.ChargeOption;
import electric_vehicles.Enums.FuelType;
import electric_vehicles.Enums.InformationStrategy;
import electric_vehicles.VehicleInfrastructure.Marketplace;
import electric_vehicles.VehicleInfrastructure.RechargeStation;
import electric_vehicles.Vehicles.Vehicle;
/**
 * @author Ayla
 *
 */
public class MemoryOwn {
	
	private int MIN_SATISFACTION_TICKS = 10;				// # ticks vehicle must be in possession until agent can be unsatisfied
	private float SATISFIED_THRESHOLD = 0.4f;				// Prop. of ticks agents must be satisfied to be flagged as satisfied

	private int d_count;
	
	private Map<String, VehicleConcept> d_vehicle_concepts;
	private Map<FuelType, FuelTypeConcept> d_fueltype_concepts;
		
	private int d_failedDrives;
	private float d_lastStepFailureCosts;
	private float d_lastCostsPerKm;
	private int d_weeklyDistance;
	private int d_weeklyEmissions;
	private InformationStrategy d_lastStrategy;
	
	private List<Boolean> d_satisfaction;			// Whether agent was satisfied or not during previous steps
	private float d_failureCostsSatisfaction; 		// Previous failureCosts satisfaction, used to compute new failure costs satisfaction
	private float d_lastEnergyCostsSatisfaction; 	// Last energy costs satisfaction, in case agent did not recharge during this drive
	
	private float d_cost_satisfaction;
	private float d_functionality_satisfaction;
	private float d_cost_satisfaction_variance;
	private float d_functionality_satisfaction_variance;
	
	// Constructor
	public MemoryOwn(Vehicle vehicle, float expertise)
	{	
		d_count = 0;
		
		d_vehicle_concepts = new HashMap<String, VehicleConcept>();
		d_vehicle_concepts.put(vehicle.getModel(), new VehicleConcept(vehicle));
		for (Vehicle model : Marketplace.getAllModels())
			if (model.getFuelType() != FuelType.GASOLINE && RandomHelper.nextDoubleFromTo(0, 1) < expertise)
				d_vehicle_concepts.put(model.getModel(), new VehicleConcept(model));

		d_fueltype_concepts = new HashMap<FuelType, FuelTypeConcept>();
		initialiseFuelTypeConcepts(vehicle.getFuelType(), expertise);

		d_failedDrives = 0;
		d_lastStepFailureCosts = 0;
		d_lastCostsPerKm = RechargeStation.getKmCosts(vehicle.getFuelType(), ChargeOption.SLOW);
		d_weeklyDistance = 0;
		d_weeklyEmissions = 0;
		d_lastStrategy = InformationStrategy.REPETITION;
		
		d_satisfaction = new LinkedList<Boolean>();
		d_failureCostsSatisfaction = 1;
		d_lastEnergyCostsSatisfaction = (float) (0.5 - 0.5 * Math.tanh(2 * 10 * (d_lastCostsPerKm - 0.1)));
		
		d_cost_satisfaction = 0;
		d_functionality_satisfaction = 0;
		d_cost_satisfaction_variance = 0;
		d_functionality_satisfaction_variance = 0;
	}
	
	// Getters
	public Map<String, VehicleConcept> getVehicleConcepts()		{ return d_vehicle_concepts; }
	public Map<FuelType, FuelTypeConcept> getFuelTypeConcepts()	{ return d_fueltype_concepts; }
	public float getFailedDrives()								{ return d_failedDrives; }
	public float getLastStepFailureCosts()						{ return d_lastStepFailureCosts; }
	public float getLastCostsPerKm()							{ return d_lastCostsPerKm; }
	public int getWeeklyDistance()								{ return d_weeklyDistance; }
	public int getWeeklyEmissions()								{ return d_weeklyEmissions; }
	public InformationStrategy getLastStrategy()				{ return d_lastStrategy; }
	public float getExistenceSatisfaction()						{ return (d_cost_satisfaction + d_functionality_satisfaction) / 2; }
	public float getFailureCostsSatisfaction()					{ return d_failureCostsSatisfaction; }
	public float getLastEnergyCostsSatisfaction()				{ return d_lastEnergyCostsSatisfaction; }
	
	public float getCostSatisfaction()							{ return d_cost_satisfaction; }
	public float getFunctionalitySatisfaction()					{ return d_functionality_satisfaction; }
	public float getCostUncertainty()							{ return (float) Math.min(1, (d_count <= 1 || d_cost_satisfaction == 0) ? 0 : Math.sqrt(d_cost_satisfaction_variance) / d_cost_satisfaction); }	
	public float getFunctionalityUncertainty()					{ return (float) Math.min(1, (d_count <= 1 || d_functionality_satisfaction == 0) ? 0 : Math.sqrt(d_functionality_satisfaction_variance) / d_functionality_satisfaction); }	

	
	// Setters
	public void resetSatisfaction()											{ d_satisfaction.clear(); }
	public void resetLastStepFailureCosts()									{ d_lastStepFailureCosts = 0; }
	public void resetFailedDrives()											{ d_failedDrives = 0; }
	public void resetWeeklyDistance()										{ d_weeklyDistance = 0; }
	public void resetWeeklyEmissions()										{ d_weeklyEmissions = 0; }
	public void increaseFailedDrives()										{ ++d_failedDrives; }
	public void setLastCostsPerKm(float costs)								{ d_lastCostsPerKm = costs;  }
	public void setFailureCostsSatisfaction(float satisfaction)				{ d_failureCostsSatisfaction = satisfaction; }
	public void setLastEnergyCostsSatisfaction(float satisfaction)			{ d_lastEnergyCostsSatisfaction = satisfaction;  }
	public void setLastStrategy(InformationStrategy strategy)				{ d_lastStrategy = strategy; }

	// Adders
	public void addLastStepFailureCosts(float costs)						{ d_lastStepFailureCosts += costs; }
	public void addWeeklyDistance(int distance)								{ d_weeklyDistance += distance; }
	public void addWeeklyEmissions(int emissions)							{ d_weeklyEmissions += emissions; }

	public void updateExistenceNeeds(float cost_satisfaction, float functionality_satisfaction)
	{
		d_count++;
		if  (d_count == 1)
		{
			d_cost_satisfaction = cost_satisfaction;
			d_functionality_satisfaction = functionality_satisfaction;
			return;
		}

		float delta = cost_satisfaction - d_cost_satisfaction;
		d_cost_satisfaction = d_cost_satisfaction * 0.9f + cost_satisfaction * 0.1f;
		d_cost_satisfaction_variance = d_cost_satisfaction_variance * 0.9f + (delta * (cost_satisfaction - d_cost_satisfaction)) * 0.1f;
		
		delta = functionality_satisfaction - d_functionality_satisfaction;
		d_functionality_satisfaction = d_functionality_satisfaction * 0.9f + functionality_satisfaction * 0.1f;
		d_functionality_satisfaction_variance = d_functionality_satisfaction_variance * 0.9f + (delta * (functionality_satisfaction - d_functionality_satisfaction)) * 0.1f;	
	}
	
	public void addSatisfaction(boolean satisfaction)
	{
		if (d_satisfaction.size() > MIN_SATISFACTION_TICKS)
			d_satisfaction.remove(0);
		d_satisfaction.add(satisfaction);
	}
	
	public void changedOwnVehicle(Vehicle vehicle)
	{
		d_count = 0;
		
		// Clear all memories of previous satisfaction levels
		resetSatisfaction();	
		d_cost_satisfaction = 0;
		d_functionality_satisfaction = 0;
		d_cost_satisfaction_variance = 0;
		d_functionality_satisfaction_variance = 0;
		
		// Clear all memories of previous failure costs
		resetLastStepFailureCosts();
		resetFailedDrives();
		setFailureCostsSatisfaction(1);		
	}
	
	public boolean satisfiedWithCurrentVehicle()
	{
		if (d_satisfaction.size() < MIN_SATISFACTION_TICKS)
			return true;
		
		float total_satisfaction = 0;
		for (boolean i : d_satisfaction)
			total_satisfaction += i ? 1 : 0;
		// If agent has been more unsatisfied than satisfied, agent demands new car.
		return (float)total_satisfaction / d_satisfaction.size() >= SATISFIED_THRESHOLD ? true : false;
	}
	
	
	private void initialiseFuelTypeConcepts(FuelType own_type, float expertise)
	{
		for (FuelType type : FuelType.values())
			d_fueltype_concepts.put(type, new FuelTypeConcept(type, type == own_type? 1 : expertise));
	}
	


}
