/**
 * 
 */
package electric_vehicles.Vehicles;

import java.util.Map;

import repast.simphony.random.RandomHelper;
import electric_vehicles.Enums.ChargeLocation;
import electric_vehicles.Enums.ChargeOption;
import electric_vehicles.Enums.FuelType;
import electric_vehicles.VehicleInfrastructure.RechargeStation;

/**
 * @author Ayla
 *
 */
public class ElectricVehicle extends Vehicle {
	
	private float d_batteryPrice;			// Proportion of price devoted to battery
	private int d_original_range;			// Distance the EV could originally travel
	private float d_chargeCapacityFast;		// Max capacity load when using fast charge (e.g. 0.8)
	private float d_max_range_decrease;		// Capacity decrease until battery is flagged as broken 
	private float d_battery_life_slow;		// Average battery life when only using slow charge
	private float d_battery_life_fast;		// Average battery life when only using fast charge
	
	public float getBatteryPrice()			{ return d_batteryPrice; }
	
	// Constructor when a new EV model is introduced
	public ElectricVehicle(FuelType fuelType, String model, int introductionTick, float price, float batteryPrice, int emissions, int capacity)
	{
		super(fuelType, model, introductionTick, price, (int)(capacity / Battery.getFuelEconomyFactor()), emissions); 
				
		d_model += "_" + capacity;
		
		d_batteryPrice = batteryPrice;
		d_original_range = (int)(capacity / Battery.getFuelEconomyFactor());
		d_capacity = capacity;
		d_chargeCapacityFast = Battery.getChargeCapacityFast();
		d_max_range_decrease = Battery.getMaxRangeDecrease();
		d_battery_life_slow = Battery.getBatteryLifeSlow();
		d_battery_life_fast = Battery.getBatteryLifeFast();
				
	}	
	
	/* @see electric_vehicles.Vehicle#copy()
	 * Copy function when an EV model is bought (copied) from new or lease market
	 */
	public ElectricVehicle copy()
	{
		return new ElectricVehicle(d_fuelType, d_model, d_introductionTick, d_price, d_batteryPrice, d_emissions, d_failures, d_capacity, d_original_range, d_chargeCapacityFast, d_max_range_decrease, d_battery_life_slow, d_battery_life_fast);
	}
	
	// Constructor when an EV model is bought (copied) from new or lease market
	public ElectricVehicle(FuelType fuelType, String model, int introductionTick, float price, float batteryPrice, int emissions, Map<Integer, Float> failures, int capacity, int original_range, float chargeCapacityFast, float max_range_decrease, float battery_life_slow, float battery_life_fast )
	{
		super(fuelType, model, model, introductionTick, price, original_range, emissions, failures);
		
		d_original_range = original_range;
		d_capacity = capacity;
		d_batteryPrice = batteryPrice;
		d_chargeCapacityFast = chargeCapacityFast;
		d_max_range_decrease = max_range_decrease;
		d_battery_life_slow = battery_life_slow;
		d_battery_life_fast = battery_life_fast;
	}
	
	/* Prints all vehicle information in a nice format
	 * @see electric_vehicles.Vehicle#printVehicle()
	 */
	public String printVehicle()
	{
		return (d_fuelType + "\t" + d_model + "\t" + d_price + "\t\t" + d_range + "\t" + d_emissions + "\t\t" + d_totalKmCount + "\t\t" + d_energy + "\t" + getValue() + "\t" + d_totalFailureCosts);
	}
	
	public boolean batteryBroken()
	{
		if (d_range < d_original_range * (1 - d_max_range_decrease))
			return true;
		return false;
	}
	
	/* On top of Vehicle.getDriveLog(), this method determines specific attributes for the EV.
	 * @see electric_vehicles.Vehicle#getDriveLog(int)
	 */
	public DriveLog getDriveLog(int total_distance, boolean workCharge, boolean homeCharge, float recharge_moment)
	{		
		if (d_fuelType == FuelType.GASOLINE)
			System.out.println("ERROR: Using Electric vehicle's method to drive a fuel car: " + printVehicle());
		
		DriveLog log = super.getDriveLog(total_distance, workCharge, homeCharge, recharge_moment);
				
		// When driving further than the current energy level of car allows, refill in between
		while (log.d_remainingKm > d_energy - d_range * recharge_moment)
		{
			//Debug.DEBUG_DRIVE(Debug.debug_agent, "Distance remaining: " + log.d_remainingKm + ". Energy: " + d_energy);

			// If this is a trip to work, slow charge halfway (at work)
			if(workCharge && ( (total_distance / 2) - (total_distance - log.d_remainingKm) ) < d_energy - d_range * recharge_moment)
			{
				//Debug.DEBUG_DRIVE(Debug.debug_agent, "Distance to work: " + ((total_distance / 2) - (total_distance - log.d_remainingKm)) + ". Energy: " + d_energy + ". Distance driven: " + (total_distance - log.d_remainingKm));
				log = chargeAtWork((total_distance / 2) - (total_distance - log.d_remainingKm), log);
				workCharge = false;
				continue;
			}
			
			log = chargeFastOnTheRoad(log, recharge_moment);
			
		}
		// When rest of the drive can be made without refilling, just subtract needed energy from current energy
		if (log.d_remainingKm > 0)
			d_energy -= log.d_remainingKm;
		
		// At the end of the drive, EV is slowly recharged at home.
		if (homeCharge)
			log = chargeAtHome(log);
		
		d_currentKmCosts = log.d_refillCosts == 0 ? d_currentKmCosts : log.d_refillCosts / log.d_refillEnergy;
		
		return log;
	}	

	
	/* Refills the battery up to a certain proportion of the max range and returns the costs of the refill
	 * Also determines whether range decreases, proportional to how much was charged.
	 * With fast charging, battery should stay above 80% capacity for +- 5 years (10k km/y)
	 */
	public int refillFast()
	{		
		int newEnergy = (int)(d_range * d_chargeCapacityFast) - d_energy;
		d_energy += newEnergy;
		
		if (RandomHelper.nextDoubleFromTo(0, 1) <= ( newEnergy / (float)(10000 * d_battery_life_fast) * (d_original_range * d_max_range_decrease ) ) )
			d_range -= 1;
		
		return newEnergy;		
	}
	
	/* Refills the battery completely using a slow charger and returns the costs of the refill
	 * Also determines whether range decreases, proportional to how much was charged.
	 * With slow charging, battery should stay above 80% capacity for +- 20 years (10k km/y)
	 */
	public int refillSlow()
	{
		int newEnergy = d_range - d_energy;
		d_energy += newEnergy;
		

		if (RandomHelper.nextDoubleFromTo(0, 1) <= ( newEnergy / (float)(10000 * d_battery_life_slow) * (d_original_range * d_max_range_decrease ) ) )
			d_range -= 1;
		
		return newEnergy;
	}
	
	/* Slow charge vehicle at home at the end of the day. No lateness is calculated
	 * because this charge is not part of the drive trip.
	 */
	public DriveLog chargeAtHome(DriveLog log)
	{
		int newEnergy = refillSlow();
		log.d_refillEnergy += newEnergy;
		log.d_refillCosts += newEnergy * RechargeStation.getKmCosts(ChargeOption.SLOW);
		return log;
	}
	
	/* Slow charge vehicle at work halfway of the drive trip. No lateness is calculated
	 * because this charge is not part of the drive trip.
	 */
	public DriveLog chargeAtWork(int distance, DriveLog log)
	{
		log.d_remainingKm -= distance;													// Drive to halfway point (work)
		d_energy -= distance;							
		int newEnergy = refillSlow();													// Slow recharge at work
		log.d_refillEnergy += newEnergy;												// Store amount of energy gained
		log.d_refillCosts += newEnergy * RechargeStation.getKmCosts(ChargeOption.SLOW);	// Store costs of new energy
		return log;
	}
	
	/* Fast charge vehicle when energy is depleted. 
	 */
	public DriveLog chargeFastOnTheRoad(DriveLog log, float recharge_moment)
	{
		log.d_remainingKm -= (int)(d_energy - d_range * recharge_moment);				// Drive as far as possible with current energy
		d_energy -= (int)(d_energy - d_range * recharge_moment);						// Set tank to empty
		int newEnergy = refillFast();													// Refill tank and keep track of gained energy
		log.d_refillEnergy += newEnergy;												// Store amount of energy gained
		log.d_refillCosts += newEnergy * RechargeStation.getKmCosts(ChargeOption.FAST);	// Store costs of new energy
		log.d_minutesLate += RechargeStation.getChargeTime(ChargeOption.FAST);			// Increase lateness
		RechargeStation.updateFastChargedEnergy(newEnergy);
		return log;
	}
	
	public float getNewBatteryCosts()
	{
		return d_capacity * Battery.getKWhCosts();
	}
	
	public void buyNewBattery()
	{		
		d_range = (int)(d_capacity / Battery.getFuelEconomyFactor());
		d_original_range = d_range;
		d_energy = d_range;
		
		d_chargeCapacityFast = Battery.getChargeCapacityFast();
		d_max_range_decrease = Battery.getMaxRangeDecrease();
		d_battery_life_slow = Battery.getBatteryLifeSlow();
		d_battery_life_fast = Battery.getBatteryLifeFast();
	}
	
	/* Check whether agent's drive plan will succeed (i.e. will agent find enough
	 * charge stations)
	 */
	public boolean driveSucceeds(int total_distance, boolean workTrip, float recharge_moment)
	{
		float remainingKm = total_distance;
		float energy = d_energy;
		
		boolean roadCharge = RandomHelper.nextDoubleFromTo(0, 1) <= RechargeStation.getChargeProbability(ChargeLocation.HIGHWAY);
		if (roadCharge)
			return true;
		
		while (remainingKm > (int)(energy - d_range * recharge_moment))
		{
			// If this is a trip to work, slow charge halfway (at work)
			if(workTrip && ( (total_distance / 2) - (total_distance - remainingKm) ) < (int)(energy - d_range * recharge_moment))
			{
				float distance_to_work = (total_distance / 2) - (total_distance - remainingKm);
				remainingKm -= distance_to_work;									// Drive to halfway point (work)
				energy -= distance_to_work;							
				energy += (d_range - energy);										// Slow recharge at work
				workTrip = false;
				continue;
			}
			
			// Check if a recharge station is available
			if (!roadCharge)
				return false;
		}
		return true;
	}
}
