/**
 * 
 */
package electric_vehicles.Vehicles;

import java.util.Map;

import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.random.RandomHelper;
import electric_vehicles.Enums.ChargeLocation;
import electric_vehicles.Enums.ChargeOption;
import electric_vehicles.Enums.FuelType;
import electric_vehicles.VehicleInfrastructure.Marketplace;
import electric_vehicles.VehicleInfrastructure.RechargeStation;

/**
 * @author Ayla
 *
 */
public class HybridVehicle extends Vehicle 
{

	private int d_capacity;
	private float d_batteryPrice;				// Proportion of price devoted to battery
	private float d_chargeCapacityFast;
	private int d_original_electric_range;
	private int d_electric_range;
	private int d_electric_energy;
	
	private float d_max_range_decrease;			// Capacity decrease until battery is flagged as broken 
	private float d_battery_life_slow;			// Average battery life when only using slow charge
	private float d_battery_life_fast;			// Average battery life when only using fast charge
	
	public int d_gasolineEmissions;
	
	public float getBatteryPrice()			{ return d_batteryPrice; }
	public int getGasolineEmissions()		{ return d_gasolineEmissions; }
	
	// Constructor when a new PHEV model is introduced
	public HybridVehicle(FuelType fuelType, String model, int introductionTick, float price, float batteryPrice, int range, int emissions, int capacity)
	{
		super(fuelType, model, introductionTick, price, range, emissions);
		
		d_model += "_" + capacity;
		
		d_batteryPrice = batteryPrice;
		d_original_electric_range = (int)(capacity / Battery.getFuelEconomyFactor());
		d_electric_range = d_original_electric_range;
		d_electric_energy = d_original_electric_range;
		
		d_capacity = capacity;
		d_chargeCapacityFast = Battery.getChargeCapacityFast();
		d_max_range_decrease = Battery.getMaxRangeDecrease();
		d_battery_life_slow = Battery.getBatteryLifeSlow();
		d_battery_life_fast = Battery.getBatteryLifeFast();		
		
		d_gasolineEmissions = (int)Marketplace.getAverageGasolineEmissions();
	}	
	
	/* @see electric_vehicles.Vehicle#copy()
	 * Copy function when an PHEV model is bought (copied) from new or lease market
	 */
	public HybridVehicle copy()
	{
		return new HybridVehicle(d_fuelType, d_model_series, d_model, d_introductionTick, d_price, d_batteryPrice, d_range, d_emissions, d_failures, d_capacity, d_original_electric_range, d_chargeCapacityFast, d_max_range_decrease, d_battery_life_slow, d_battery_life_fast, d_gasolineEmissions);
	}
	
	// Constructor when an PHEV model is bought (copied) from new or lease market 
	public HybridVehicle(FuelType fuelType, String model_series, String model, int introductionTick, float price, float batteryPrice, int range, int emissions, Map<Integer, Float> failures, int capacity, int original_electric_range, float chargeCapacityFast, float max_range_decrease, float battery_life_slow, float battery_life_fast, int gasolineEmissions )
	{
		super(fuelType, model_series, model, introductionTick, price, range, emissions, failures); // 0.5 p/50km, 0.5 p/50km, 1 p/50km --> 2.00 p/50km
		
		d_batteryPrice = batteryPrice;
		d_original_electric_range = original_electric_range;
		d_electric_range = original_electric_range;
		d_electric_energy = original_electric_range;
		
		d_capacity = capacity;
		d_chargeCapacityFast = chargeCapacityFast;
		d_max_range_decrease = max_range_decrease;
		d_battery_life_slow = battery_life_slow;
		d_battery_life_fast = battery_life_fast;
		d_gasolineEmissions = gasolineEmissions;
	}
	
	public int getRange()					{ return d_range + d_electric_range; }
	
	/* Prints all vehicle information in a nice format
	 * @see electric_vehicles.Vehicle#printVehicle()
	 */
	public String printVehicle()
	{
		return (d_fuelType + "\t" + d_model + "\t" + d_price + "\t\t" + d_range + "\t" + d_emissions + "\t\t" + d_totalKmCount + "\t\t" + d_energy + "\t" + getValue() + "\t" + d_totalFailureCosts);
	}
	
	public boolean batteryBroken()
	{
		if (d_range < d_original_electric_range * (1 - d_max_range_decrease))
			return true;
		return false;
	}
	
	public void setPrice()
	{
		d_price += d_capacity * Battery.getKWhCosts();
	}
	
	/* On top of Vehicle.getDriveLog(), this method determines specific drive attributes for a Hybrid.
	 * @see electric_vehicles.Vehicle#getDriveLog(int)
	 */
	public DriveLog getDriveLog(int total_distance, boolean workCharge, boolean homeCharge, float recharge_moment)
	{
		DriveLog log = super.getDriveLog(total_distance, workCharge, homeCharge, recharge_moment);
				
		// When driving further than the current energy level of car allows, refill in between
		while (log.d_remainingKm > (d_electric_energy + (d_energy - d_range * recharge_moment)))
		{
			// If this is a trip to work and work can be reached electrically, slow charge halfway (at work)
			if (workCharge && ( (total_distance / 2) - (total_distance - log.d_remainingKm) ) < d_electric_energy + (d_energy - d_range * recharge_moment))
			{
				log = chargeAtWork((total_distance / 2) - (total_distance - log.d_remainingKm), log);
				workCharge = false;
				continue;
			}
						
			// Else, drive electrically as far as possible, then switch to fuel
			log.d_remainingKm -= d_electric_energy;
			log.d_electricKmWithPHEV += d_electric_energy;
			d_electric_energy = 0;
			refuel(log, recharge_moment);			
		}
		// When rest of the drive can be made without refilling, subtract needed energy first from electric energy, then from fuel energy
		if (log.d_remainingKm > 0 && d_electric_energy >= log.d_remainingKm)
		{
			log.d_electricKmWithPHEV += log.d_remainingKm;
			d_electric_energy -= log.d_remainingKm;
		}
		else if (log.d_remainingKm > 0)
		{
			if (workCharge)
				log = chargeAtWork((total_distance / 2), log);
			int remaining_distance = log.d_remainingKm - d_electric_energy;
			log.d_electricKmWithPHEV += d_electric_energy;
			d_electric_energy = 0;
			d_energy -= remaining_distance;
		}
			
		// At the end of the drive, EV-part is slowly recharged at home.
		if (homeCharge)
			log = chargeAtHome(log);
		
		d_currentKmCosts = log.d_refillCosts == 0 ? d_currentKmCosts : log.d_refillCosts / log.d_refillEnergy;
		
		return log;
	}	
	
	/* Refills the battery up to a certain proportion of the max range and returns the costs of the refill
	 * Also determines whether range decreases, proportional to how much was charged.
	 * With fast charging, battery should stay above 80% capacity for +- 5 years (10k km/y)
	 */
	public int refillFast()
	{		
		int newEnergy = (int)(d_electric_range * d_chargeCapacityFast) - d_electric_energy;
		d_electric_energy += newEnergy;
		
		if (RandomHelper.nextDoubleFromTo(0, 1) <= ( newEnergy / (float)(10000 * d_battery_life_fast) * (d_original_electric_range * d_max_range_decrease ) ) )
			d_electric_range -= 1;
		
		return newEnergy;		
	}
	
	/* Refills the battery completely using a slow charger and returns the costs of the refill
	 * Also determines whether range decreases, proportional to how much was charged.
	 * With slow charging, battery should stay above 80% capacity for +- 20 years (10k km/y)
	 */
	public int refillSlow()
	{
		int newEnergy = d_electric_range - d_electric_energy;
		d_electric_energy += newEnergy;
		

		if (RandomHelper.nextDoubleFromTo(0, 1) <= ( newEnergy / (float)(10000 * d_battery_life_slow) * (d_original_electric_range * d_max_range_decrease ) ) )
			d_electric_range -= 1;
		
		return newEnergy;
	}
	
	/* Refills the tank completely and returns the costs of the refill
	 */
	public int refillFuel()
	{	
		int newEnergy = d_range - d_energy;
		d_energy += newEnergy;
		return newEnergy;		
	}
	
	/* Slow charge vehicle at home at the end of the day. No lateness is calculated
	 * because this charge is not part of the drive trip.
	 */
	public DriveLog chargeAtHome(DriveLog log)
	{
		int newEnergy = refillSlow();
		log.d_refillEnergy += newEnergy;
		log.d_refillCosts += newEnergy * RechargeStation.getKmCosts(ChargeOption.SLOW);
		return log;
	}
	
	/* Slow charge vehicle at work halfway of the drive trip. No lateness is calculated
	 * because this charge is not part of the drive trip.
	 */
	public DriveLog chargeAtWork(int distance, DriveLog log)
	{
		log.d_remainingKm -= distance;													// Drive to halfway point (work)
		int fuel_km = Math.max(0, distance - d_electric_energy);						// Use electric as much as possible
		log.d_electricKmWithPHEV += Math.max(0, d_electric_energy - distance);
		d_electric_energy = Math.max(0, d_electric_energy - distance);
		d_energy -= fuel_km;
		
		if (d_energy < 0)
			System.out.println("ERROR: chargeAtWork() in Hybrid class. Negative energy.");
		
		int newEnergy = refillSlow();													// Slow recharge at work
		log.d_refillEnergy += newEnergy;												// Store amount of energy gained
		log.d_refillCosts += newEnergy * RechargeStation.getKmCosts(ChargeOption.SLOW);	// Store costs of new energy
		return log;
	}
	
	/* Refill fuel when energy is depleted.
	 */
	public DriveLog refuel(DriveLog log, float recharge_moment)
	{
		log.d_remainingKm -= (int)(d_energy - d_range * recharge_moment);								// Drive as far as possible with current fuel
		d_energy -= (int)(d_energy - d_range * recharge_moment);										// Reduce fuel in tank
		int newEnergy = refillFuel();																	// Refill tank and keep track of how often tank is refilled
		log.d_refillEnergy += newEnergy;																// Store amount of energy gained
		log.d_refillCosts += newEnergy * RechargeStation.getKmCosts(ChargeOption.GASOLINE);				// Store costs of new energy
		log.d_minutesLate += RechargeStation.getChargeTime(ChargeOption.GASOLINE);						// Increase lateness
		return log;
	}
	
	public float getNewBatteryCosts()
	{
		return d_capacity * Battery.getKWhCosts();
	}
	
	public void buyNewBattery()
	{		
		d_electric_range = (int)(d_capacity / Battery.getFuelEconomyFactor());
		d_original_electric_range = d_electric_range;
		d_electric_energy = d_electric_range;
		
		d_chargeCapacityFast = Battery.getChargeCapacityFast();
		d_max_range_decrease = Battery.getMaxRangeDecrease();
		d_battery_life_slow = Battery.getBatteryLifeSlow();
		d_battery_life_fast = Battery.getBatteryLifeFast();
	}
	
}
