/**
 * 
 */
package electric_vehicles.Vehicles;

import java.util.Map;

import repast.simphony.essentials.RepastEssentials;
import repast.simphony.random.RandomHelper;
import electric_vehicles.Enums.ChargeOption;
import electric_vehicles.Enums.FuelType;
import electric_vehicles.VehicleInfrastructure.Mechanic;
import electric_vehicles.VehicleInfrastructure.RechargeStation;

/**
 * @author Ayla
 *
 */
public class Vehicle {
	
	// Vehicle properties
	protected FuelType d_fuelType;
	protected String d_model_series;
	protected String d_model;
	protected int d_introductionTick;
	protected int d_firstPurchaseTick;
	protected float d_price;
	protected int d_range;
	protected int d_capacity = 0;						// Capacity of battery in kWh, only used by EVs and PHEVs
	protected int d_emissions;
	
	// Aspects related to car failure
	protected Map<Integer, Float> d_failures;		// Key: Failure cost, Value: Failure probability
	protected float d_remainingFailureKm;
	
	protected int d_purchaseTick;
	
	// Vehicle aspects that differ over time
	protected int d_totalKmCount;
	protected int d_energy;
	protected float d_currentKmCosts;
	protected float d_totalFailureCosts;
	
	// Constructor when a new vehicle model is introduced
	public Vehicle(FuelType fuelType, String model, int introductionTick, float price, int range, int emissions)
	{
		d_fuelType = fuelType;
		d_model_series = model;
		d_model = model;
		d_introductionTick = introductionTick;
		d_price = price;
		d_range = range;
		d_emissions = emissions;
		
		d_failures = Mechanic.getFailureProbabilities(d_fuelType);
		d_remainingFailureKm = 0;
		
		d_purchaseTick = (int)RepastEssentials.GetTickCount();
		d_firstPurchaseTick = d_purchaseTick;

		d_totalKmCount = 0;
		d_energy = d_range; 
		d_currentKmCosts = RechargeStation.getKmCosts(d_fuelType, ChargeOption.SLOW);
		d_totalFailureCosts = 0;
	}
	
	//Copy function when a vehicle is bought (copied) from new or lease market
	public Vehicle copy()
	{
		return new Vehicle(d_fuelType, d_model_series, d_model, d_introductionTick, d_price, d_range, d_emissions, d_failures);
	}
	
	// Constructor when a vehicle is bought (copied) from new or lease market
	public Vehicle(FuelType fuelType, String model_series, String model, int introductionTick, float price, int range, int emissions,  Map<Integer, Float> failures)
	{
		d_fuelType = fuelType;
		d_model_series = model_series;
		d_model = model;
		d_introductionTick = introductionTick;
		d_price = price;
		d_range = range;
		d_emissions = emissions;
		
		d_failures = failures;
		d_remainingFailureKm = 0;
		
		d_purchaseTick = (int)RepastEssentials.GetTickCount();
		d_firstPurchaseTick = d_purchaseTick;

		d_totalKmCount = 0;
		d_energy = d_range; 
		d_currentKmCosts = RechargeStation.getKmCosts(d_fuelType, ChargeOption.SLOW);
		d_totalFailureCosts = 0;
	}
	
	// GETTERS
	public FuelType getFuelType()			{ return d_fuelType; }
	public String getModel()				{ return d_model; }
	public String getModelSeries()			{ return d_model_series; }
	public int getIntroductionTick()		{ return d_introductionTick; }
	public float getPrice()					{ return d_price; }
	public float getBatteryPrice()			{ System.out.println("ERROR: getBatteryPrice should not be called for fuel type " + d_fuelType); return 0; }
	public int getRange()					{ return d_range; }
	public int getCapacity()				{ return d_capacity; }
	public Map<Integer, Float> getFailures(){ return d_failures; }	
	public float getAverageFailureCosts()	{ return d_totalFailureCosts < 500 ? Mechanic.getAverageFailureCostsPerKm(d_fuelType) : d_totalFailureCosts / d_totalKmCount; }
	public float getCurrentKmCosts()		{ return d_currentKmCosts; }
	
	public float getValue()					{ return (float) (Math.pow(0.85, ((RepastEssentials.GetTickCount() - d_firstPurchaseTick) / 52) ) * d_price); }
	public int getKmCount()					{ return d_totalKmCount; }
	public int getEnergy()					{ return d_energy; }
	public float getFailureCosts()			{ return d_totalFailureCosts; }
	public float getPurchaseTick()			{ return d_purchaseTick; }
	public float getFirstPurchaseTick()		{ return d_firstPurchaseTick; }
	public int getEmissions()				{ return d_emissions; }
	public int getGasolineEmissions()		{ return d_emissions; }	// See Hybrid for overloaded function
	public boolean batteryBroken()			{ return false;}		// See Electric & Hybrid for different implementations
	
	//SETTERS
	public void resetTotalFailureCosts()	{ d_totalFailureCosts = 0; }
	public void resetEnergy()				{ d_energy = d_range; }
	public void resetPurchaseTick()			{ d_purchaseTick = (int)RepastEssentials.GetTickCount(); }
	
	// Only set upon initialization
	public void setKmCount(int km)				{ d_totalKmCount = km; }
	public void setIntroductionTick(int tick)	{ d_introductionTick = tick; } 
	public void setPurchaseTick(int tick)		{ d_purchaseTick = tick; }
	public void setFirstPurchaseTick(int tick)	{ d_firstPurchaseTick = tick; }
	public void setFailureCosts(float costs)	{ d_totalFailureCosts += costs; }
	
	/* Prints all vehicle information in a nice format
	*/
	public String printVehicle()
	{
		return (d_fuelType + "\t" + d_model + "\t" + d_price + "\t\t" + d_range + "\t" + d_emissions + "\t\t" + d_totalKmCount + "\t\t" + d_energy + "\t" + getValue() + "\t" + d_totalFailureCosts);
	}
	
	/* This function is always called by its inherited classes, see GasolineVehicle and ElectricVehicle.
	 * Updates the km count and fuel level after the user has taken the car for a drive.
	 * Also determines whether a car failure has taken place.
	 */
	public DriveLog getDriveLog(int distance, boolean workCharge, boolean homeCharge, float recharge_moment)
	{
		DriveLog log = new DriveLog(distance);		
		
		// Obtain the remaining km of past rides over which no failure probability was calculated yet
		d_remainingFailureKm = d_totalKmCount % getFailureKmUnit();
		
		// Increase the total km count of the car
		d_totalKmCount += distance;		
		
		// Determine whether random failure(s) occur(s)
		// Calculate failure probability over current distance and remaining distance from previous rides
		float failureKm = log.d_distance + d_remainingFailureKm;
		// Each 'd_failureKmUnit', there is a probability of failure
		int failureCount = (int)Math.floor(failureKm / getFailureKmUnit());
		for (int i = 0; i < failureCount; ++i)
		{
			if (RandomHelper.nextDoubleFromTo(0, 1) <= d_failures.get(50))
				log.d_failureCosts += 50;
			if (RandomHelper.nextDoubleFromTo(0, 1) <= d_failures.get(500))
				log.d_failureCosts += 500;
			if (RandomHelper.nextDoubleFromTo(0, 1) <= d_failures.get(2000))
				log.d_failureCosts += 2000;
		}
		d_totalFailureCosts += log.d_failureCosts;
		
		//log.d_minutesLate and d_refillCosts are set by inherited classes
		return log;
	}
	
	int getFailureKmUnit()
	{
		return Math.max(40, 55 - Math.max((d_totalKmCount / 13000), ((int)RepastEssentials.GetTickCount() - d_firstPurchaseTick)/52));
	}
	
	
	public boolean driveSucceeds(int total_distance, boolean workTrip, float recharge_moment)
	{
		// See children classes
		System.out.println("ERROR: Parent function driveSucceeds(int total_distance, boolean workTrip, float recharge_moment) is called.");
		return true;
	}
	
	public float getNewBatteryCosts()
	{
		// See children classes
		System.out.println("ERROR: Parent function getNewBatteryCosts() is called.");
		return 0;
	}
	
	public void buyNewBattery()
	{
		// See children classes
		System.out.println("ERROR: Parent function buyNewBattery() is called.");
	}
	
	public void updateCapacity(int capacity)
	{
		// See children classes
		System.out.println("ERROR: Parent function updateCapacity() is called.");
	}
	
	public void updateEmissions(int emissions)
	{
		// See children classes
		System.out.println("ERROR: Parent function updateEmissions() is called.");
	}
	
}
